@@ -7,8 +7,9 @@ use Digest::MD5 qw(md5);
 use Fcntl qw(:flock);
 use IO::Handle;
 use POSIX qw(:fcntl_h);
+use File::Path;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 use Class::Accessor::Lite (
     ro => [ qw(base_dir worker_id) ],
@@ -21,7 +22,7 @@ sub new {
         unless $args{base_dir};
     # create base_dir if necessary
     if (! -e $args{base_dir}) {
-        mkdir $args{base_dir}
+	mkpath $args{base_dir}
             or die "failed to create directory:$args{base_dir}:$!";
     }
     # build object
@@ -40,8 +41,8 @@ sub DESTROY {
     # if file is open, close and unlink
     if ($self->{fh}) {
         close $self->{fh};
-        my $fn = $self->_build_filename();
-        unlink $fn;
+        # during global destruction we may already have lost this
+	unlink $self->_build_filename() if ($self->{base_dir});
     }
 }
 
